from PyQt5 import QtWidgets,uic
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from src.dialogError import dError
from src.Interpolation import interpolationPassage,interpolationPassageFreq
import pandas as pd


path_pkl = 'src/data_experiments/'
path_buttons='src/buttons_and_background/'
class dInterpolation(QtWidgets.QDialog):
    def __init__(self, experiments,parent=None):
        super(dInterpolation, self).__init__()
        uic.loadUi('src/ui_files/interpolateDialog.ui', self)
        self._flag = False
        self.pushButton.clicked.connect(self.interpolation)
        self.model2 = QStandardItemModel()
        self.model2.setHorizontalHeaderLabels(['Experiment '])
        self.tableView.setModel(self.model2)
        self.tableView.horizontalHeader().setSectionResizeMode(1)
        self.experiments = experiments
        self.pushButton.setStyleSheet("QPushButton{border-image:url("+path_buttons+"Ok.png);}")
        self.pushButton.setToolTip("Click this button to interpolate experiment")
        for df in experiments:
            experimentName = "Strain: " + str(df[0]) + " " + "Repetition: " + str(df[1])
            row = (QStandardItem(experimentName))
            self.model2.appendRow(row)
        self.executed = False
        self.typeInterpolation = ''
        self.lvp = ''
        self.fvp = ''
        self.strain = ''
        self.repetition = ''
        self.passage = 0

    def interpolation(self):
        indices = self.tableView.selectionModel().selectedRows()
        if len(indices) < 1:

            de = dError('Error: Please select one experiment')
            de.exec()
        elif len(indices) > 1:

            de = dError('Error: Please select only one experiment')
            de.exec()
        elif (self.EntropyB.isChecked()==False and self.FrequencyB.isChecked()==False):

            de = dError('Error: Please select type of interpolation')
            de.exec()
        else:
            try:

                self.passage = int(self.textEdit.toPlainText())

                for index in sorted(indices):
                    experimentIndex = (index.row())

                df = self.experiments[experimentIndex]
                self.strain = str(df[0])
                self.repetition = str(df[1])
                self.fvp = str(df[2])
                self.lvp = str(df[3])
                if self.EntropyB.isChecked()==True:
                    experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ')_entropy.parquet'
                    df = pd.read_parquet(path_pkl + experimentName)
                    dfAux = interpolationPassage(str(self.passage),df)
                    self.typeInterpolation = 'Entropy'
                else:
                    experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ').parquet'
                    df = pd.read_parquet(path_pkl + experimentName)
                    dfAux = interpolationPassageFreq(str(self.passage),df)
                    self.typeInterpolation = 'Frequency'
                dfAux.to_parquet(path_pkl + experimentName)
                self.executed = True
                self.close()
            except Exception as e:

                de = dError('Error: Please enter a integer passage')
                de.exec()

    def getInfo(self):
        return self.executed,self.typeInterpolation,self.strain,self.repetition,self.fvp,self.lvp,str(self.passage)




import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.manifold import TSNE
from pandas.plotting import scatter_matrix

def keepValidPositions(dfE, fvp, lvp):
    #Función que toma el dataframe y se queda solamente con las posiciones válidas
    
    #Busco el valor de las posiciones en el dataframe
    posiciones=dfE.Position.unique()
    
    #Encuentro los indices donde las posiciones son validas
    idx=np.where((posiciones>=fvp)&(posiciones<=lvp))
    
    #Guardo en posVal las posiciones validas
    posVal=posiciones[idx]
    
    #Me quedo con las posiciones validas del dataframe
    dfE_val=dfE[dfE.Position.isin(posVal)].copy()
    
    return dfE_val
    
def experimentToPassages(df):
    #Función que toma el experimento (dataframe df) y convierte cada pasaje en una fila
    
    #Busco los pasajes
    passages=df['Passage'].unique()
    
    #Obtengo la info
    info_df=str(df['info'].iloc[0])
    strain_df=info_df.split(',')[0]
    repetition_df=info_df.split(',')[1]
    
    #Busco el valor de las posiciones en el dataframe
#    posiciones=df.Position.unique()
    #Encuentro los indices donde las posiciones son validas
#    idx=np.where((posiciones>=fvp)&(posiciones<=lvp))
    #Guardo en posVal las posiciones validas
#    posVal=posiciones[idx]
    
    arrayDF_P  = []
    arrayInfo = []
    arrayStrain = []
#    print(isinstance(df,pd.DataFrame))
    
    for p in passages:
#        dfP=df[df.Passage.eq(p)]
#        dfP_val=dfP[dfP.Position.isin(posVal)].copy()
#        passageArray=np.array(dfP_val[['Entropy']])

        dfP=df[df.Passage.eq(p)]
#        print(isinstance(dfP,pd.DataFrame))
        dfPE=dfP[['Entropy']]
#        print(isinstance(dfPE,pd.DataFrame))
        passageArray=np.array(dfPE)
        passageArray=np.array(dfPE)
        
        pA = np.squeeze(np.array([passageArray]))
        arrayDF_P.append(pA)
        arrayInfo.append(str(strain_df)+','+str(repetition_df)+','+str(p))
        arrayStrain.append(str(strain_df))
    return np.array(arrayDF_P), np.array(arrayInfo), np.array(arrayStrain)

        
def totalExperimentsToPassages(parameters):
    #Función que toma todos los experimentos que haya cargados en parameters y convierte cada experimento a una fila por pasaje
    
    #Arrays donde se guardan los pasajes como filas y la info
    arrayT_DF_P  = []
    arrayT_Info = []
    arrayT_Strain = []
    
    totalArrayData = []
    totalArrayInfo = []
    totalArrayStrain = []
#    firstIteration = True
    
    for df in parameters:
        arrayP, arrayI, arrayS=experimentToPassages(df)
        arrayT_DF_P.append(arrayP)
        arrayT_Info.append(arrayI)
        arrayT_Strain.append(arrayS)
    
    totalArrayData = np.concatenate(arrayT_DF_P)
    totalArrayInfo = np.concatenate(arrayT_Info)
    totalArrayStrain = np.concatenate(arrayT_Strain)
        
    return totalArrayData, totalArrayInfo, totalArrayStrain

def totalExperimentsToPoints(parameters):
    #Función qur toma todos los experimentos cargados y los convierte en una fila (cada experimento pasa a ser una fila)
    
    #Arrays donde se guardan los pasajes como filas y la info
    arrayDF  = []
    infoDF = []
    strainDF = []
    for df in parameters:
        #Obtengo la info
        info_df=str(df['info'].iloc[0])
        strain=info_df.split(',')[0]
        arrayDF.append(df['Entropy'])
        infoDF.append(df['info'])
        strainDF.append(strain)
    aDFasPoints = np.squeeze(np.array([arrayDF]))
    strainDFasPoints=np.squeeze(np.array(strainDF))
    return aDFasPoints, strainDFasPoints

def formatS_tSNE(strainT):
    #Función que toma el array de strains y le asigna a cada una un int a partir del 0
    #Ademas calcula el valor que debe tomar el parámetro perplexity de tSNE de acuerdo a la cantidad de puntos de cada cepa
    
    #Paso a dataframe las cepas de los experimentos cargados
    df = pd.DataFrame(strainT,columns=["Strain"])
    df = df.assign(StrainNumber = df['Strain'])
    #Busco la cantidad de pasajes de cada cepa
    strainsU=df.Strain.unique()
    dataRows = []
    i=0
    for s in strainsU:
        n=df[df.Strain.eq(s)].count()
        dataRows.append(n)
        df.loc[df['Strain'] == s, 'StrainNumber'] = i
#        df[df.Strain.eq(s)]["StrainNumber"]=i
        i+=1
    arPerp = np.array(dataRows)
    p = arPerp.min()
    perp = p//3
#    print(perp)
#    print(df.shape)
    return perp, df


def apply_2D_tSNE(data, infoStrains, randState, perp):
    #Función que calcula los componentes 2D de tSNE

    transformedData = TSNE(perplexity=perp, random_state=randState).fit_transform(data)
    return transformedData

def apply_3D_tSNE(data, randState, perp):
    #Función que calcula los componentes 3D de tSNE
    
    transformedData = TSNE(n_components=3,perplexity=perp, random_state=randState).fit_transform(data)
    return transformedData


def tsne_SB_Scatter2D(datos, strainsDF):
    #Función que plotea como scatter la tSNE 2D basado en seaborn
    
    dfD = pd.DataFrame(datos,columns=["tSNE-2d-1", "tSNE-2d-2"])
#    print(datos.shape)
#    print(strainsDF.shape)
    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
#    dfS = pd.DataFrame(strainsDF,columns=["Strain"])
    dfT=pd.concat([dfD,dfS], axis=1)
#    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
#    sns.lmplot( x="X", y="Y", data=datos, fit_reg=False, hue=strainsArr, legend=False)
    sns.pairplot( x_vars="tSNE-2d-1", y_vars="tSNE-2d-2", data=dfT, hue="Strain", size=5)
#    sns.lmplot( x="tSNE-2d-1", y="tsne-2d-2", data=df, hue=strainsArr, legend=False)
#    sns.plt.show()
    return


def tSNE_SB_Pairgrid(datos, strainsDF):
    #Función que realiza el pairplot de los primeras 3 componentes de tSNE
    fig, ax = plt.subplots()
    dfD = pd.DataFrame(datos,columns=["tSNE-1", "tSNE-2", "tSNE-3"])
    dfS = pd.DataFrame(strainsDF,columns=["Strain"])
    dfT=pd.concat([dfD,dfS], axis=1)
    strains=dfT.Strain.unique()
    g=sns.PairGrid(dfT, hue="Strain")
    g = g.map(plt.scatter)
    ax.legend()
    name = 'plot_images/tSNE_Pairgrid' +str(strains)+'_'+'.png'
    plt.savefig(name)
    plt.close()
    return strains


def tSNE_SB_Pairplot(datos, strainsDF):
    #Función que realiza el pairplot de los primeras 3 componentes de tSNE
    fig, ax = plt.subplots()
    dfD = pd.DataFrame(datos,columns=["tSNE-1", "tSNE-2", "tSNE-3"])
    dfS = pd.DataFrame(strainsDF,columns=["Strain"])
    dfT=pd.concat([dfD,dfS], axis=1)
    strains=dfT.Strain.unique()
    sns.pairplot(dfT, hue="Strain")
#    g = g.map(plt.scatter)
    ax.legend()
    name = 'plot_images/tSNE_Pairplot' +str(strains)+'_'+'.png'
    plt.savefig(name)
    plt.close()
    return


def tSNE_PD_scatterM(datos, strainsDF):
    #Función que realiza el pairplot de los primeras 3 componentes de tSNE
    fig, ax = plt.subplots()
    dfD = pd.DataFrame(datos,columns=["tSNE-1", "tSNE-2", "tSNE-3"])
    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
    dfT=pd.concat([dfD,dfS], axis=1)
    strains=dfT.Strain.unique()
    scatter_matrix(dfT, c=dfT.StrainNumber)
    ax.legend()
    name = 'plot_images/tSNE_ScatterMatrix' +str(strains)+'_'+'.png'
    plt.savefig(name)
    plt.close()
#    plt.show
    return


def tSNE_Pairplot(datos, strainsDF):
    #Función que realiza el pairplot de los primeras 3 componentes de tSNE
    fig, ax = plt.subplots()
    dfD = pd.DataFrame(datos,columns=["tSNE_1", "tSNE_2", "tSNE_3"])
    dfS = pd.DataFrame(strainsDF,columns=["Strain"])
    dfT=pd.concat([dfD,dfS], axis=1)
    strains=dfT.Strain.unique()
    
    ax1=plt.subplot(331)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d.tSNE_1
        y=d.tSNE_1
        ax1.scatter(x, y, label=s)
        
    ax2=plt.subplot(332)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d.tSNE_2
        y=d.tSNE_1
        ax2.scatter(x, y, label=s)
    
    ax3=plt.subplot(333)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d['tSNE_3']
        y=d['tSNE_1']
        ax3.scatter(x, y, label=s)
        
    ax4=plt.subplot(334)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d['tSNE_1']
        y=d['tSNE_2']
        ax4.scatter(x, y, label=s)
        
    ax5=plt.subplot(335)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d['tSNE_2']
        y=d['tSNE_2']
        ax5.scatter(x, y, label=s)
        
    ax6=plt.subplot(336)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d['tSNE_3']
        y=d['tSNE_2']
        ax6.scatter(x, y, label=s)
        
    ax7=plt.subplot(337)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d['tSNE_1']
        y=d['tSNE_3']
        ax7.scatter(x, y, label=s)
        
    ax8=plt.subplot(338)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d['tSNE_2']
        y=d['tSNE_3']
        ax8.scatter(x, y, label=s)
        
    ax9=plt.subplot(339)
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d['tSNE_3']
        y=d['tSNE_3']
        ax9.scatter(x, y, label=s)
        
#    plt.show()
    ax.legend()
    name = 'plot_images/tSNE_Pairplot' +str(strains)+'_'+'.png'
    plt.savefig(name)
    plt.close()
    return


def tsne_MPL_Scatter2D(datos, strainsDF):
    #Función que plotea como scatter la tSNE 2D basado en matplotlib
    fig, ax = plt.subplots()
    dfD = pd.DataFrame(datos,columns=["tSNE_2d_1", "tSNE_2d_2"])
#    print(datos.shape)
#    print(strainsDF.shape)
    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
#    dfS = pd.DataFrame(strainsDF,columns=["Strain"])
    dfT=pd.concat([dfD,dfS], axis=1)
    strains=dfT.Strain.unique()
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d.tSNE_2d_1
        y=d.tSNE_2d_2
        ax.scatter(x, y, label=s)
    ax.legend()
    name = 'plot_images/tSNE_2D' +str(strains)+'_'+'.png'
    plt.savefig(name)
    plt.close()
#    plt.show()
#    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
#    sns.lmplot( x="X", y="Y", data=datos, fit_reg=False, hue=strainsArr, legend=False)
#    sns.pairplot( x_vars="tSNE-2d-1", y_vars="tSNE-2d-2", data=dfT, hue="Strain", size=5)
#    sns.lmplot( x="tSNE-2d-1", y="tsne-2d-2", data=df, hue=strainsArr, legend=False)
#    sns.plt.show()
    return strains


def tsne_MPL_Scatter2D_exp(datos, strainsDF):
    #Función que plotea como scatter la tSNE 2D basado en matplotlib
    #Esta sin hacer
    fig, ax = plt.subplots()
    dfD = pd.DataFrame(datos,columns=["tSNE_2d_1", "tSNE_2d_2"])
#    print(datos.shape)
#    print(strainsDF.shape)
    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
#    dfS = pd.DataFrame(strainsDF,columns=["Strain"])
    dfT=pd.concat([dfD,dfS], axis=1)
    strains=dfT.Strain.unique()
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d.tSNE_2d_1
        y=d.tSNE_2d_2
        ax.scatter(x, y, label=s)
    ax.legend()
    name = 'plot_images/tSNE_2D_exp' +str(strains)+'_'+'.png'
    plt.savefig(name)
    plt.close()
#    plt.show()
#    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
#    sns.lmplot( x="X", y="Y", data=datos, fit_reg=False, hue=strainsArr, legend=False)
#    sns.pairplot( x_vars="tSNE-2d-1", y_vars="tSNE-2d-2", data=dfT, hue="Strain", size=5)
#    sns.lmplot( x="tSNE-2d-1", y="tsne-2d-2", data=df, hue=strainsArr, legend=False)
#    sns.plt.show()
    return strains


def tsne_MPL_Scatter_3D(datos, strainsDF):
    #Función que plotea como scatter la tSNE 3D basado en matplotlib
    
    dfD = pd.DataFrame(datos,columns=["tSNE_3d_1", "tSNE_3d_2", "tSNE_3d_3"])
    dfS = pd.DataFrame(strainsDF,columns=["Strain", "StrainNumber"])
    strains=dfS.Strain.unique()
    dfT=pd.concat([dfD,dfS], axis=1)
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1, facecolor="1.0")
    ax = fig.gca(projection='3d')
    for s in strains:
        d=dfT.loc[dfT['Strain'] == s]
        x=d.tSNE_3d_1
        y=d.tSNE_3d_2
        z=d.tSNE_3d_3
        ax.scatter(x, y, z, label=s)
    ax.legend()
    name = 'plot_images/tSNE_3D' +str(strains)+'_'+'.png'
    plt.savefig(name)
    plt.close()
#    plt.show()
#    ax = fig.add_subplot(111, axisbg="1.0", projection='3d')
#    ax.scatter(x, y, z, alpha=0.8, c=color, edgecolors='none', s=30, label=strains)
#    ax.scatter(x, y, z, alpha=0.8, edgecolors='none', s=30, label=strains)
#    sns.lmplot( x="X", y="Y", data=datos, fit_reg=False, hue=strainsArr, legend=False)
#    plt.scatter( x="tSNE-3d-1", y="tSNE-3d-2", z="tSNE-3d-3")
#    sns.lmplot( x="tSNE-2d-1", y="tsne-2d-2", data=df, hue=strainsArr, legend=False)
#    sns.plt.show()
    return strains
    
    
    # choose a color palette with seaborn.
#    num_classes = len(np.unique(strainsArr))
#    txts = np.unique(strainsArr)
#    palette = np.array(sns.color_palette("hls", num_classes))

    # create a scatter plot.
#    f = plt.figure(figsize=(8, 8))
#    ax = plt.subplot(aspect='equal')
#    sc = ax.scatter(datos[:,0], datos[:,1], alpha=0.7, lw=0, s=40, label=strainsArr)
#    sc = ax.scatter(datos[:,0], datos[:,1], alpha=0.7, lw=0, s=40, c=txts, cmap=cm.brg)
#    sc = ax.scatter(datos[:,0], datos[:,1], lw=0, s=40, c=palette[strainsArr.astype(np.int)])
#    plt.xlim(-25, 25)
#    plt.ylim(-25, 25)
#    ax.axis('off')
#    ax.axis('tight')

    # add the labels for each digit corresponding to the label
    
#    txts = []
    
#    for i in range(num_classes):
#
#        # Position of each label at median of data points.
#
#        xtext, ytext = np.median(datos[strainsArr == i, :], axis=0)
#        txt = ax.text(xtext, ytext, str(i), fontsize=24)
#        txt.set_path_effects([
#            PathEffects.Stroke(linewidth=5, foreground="w"),
#            PathEffects.Normal()])
#        txts.append(txt)

#    return f, ax, sc, txts

def apply_tSNE_2D_SB_passages(experimentsP,randSeed):
    
    todosLosPasajes,todaLaInfo, todasS = totalExperimentsToPassages(experimentsP)
    perplex, tStrain = formatS_tSNE(todasS)
    tDTotal2D = apply_2D_tSNE(todosLosPasajes,tStrain,randSeed, perplex)
    tsne_SB_Scatter2D(tDTotal2D,tStrain)
    return


def apply_tSNE_2D_passages(experimentsP,randSeed):
    
    todosLosPasajes,todaLaInfo, todasS = totalExperimentsToPassages(experimentsP)
    perplex, tStrain = formatS_tSNE(todasS)
    tDTotal2D = apply_2D_tSNE(todosLosPasajes,tStrain,randSeed, perplex)
    strains = tsne_MPL_Scatter2D(tDTotal2D,tStrain)
    return strains


def apply_tSNE_2D_pairgrid_passages(experiments,randSeed):
    
    todosLosPasajes,todaLaInfo, todasS = totalExperimentsToPassages(experiments)
    perplex, tStrain = formatS_tSNE(todasS)
    tDTotal3D = apply_3D_tSNE(todosLosPasajes,randSeed,perplex)
    strains = tSNE_SB_Pairgrid(tDTotal3D,todasS)
    return strains

def apply_tSNE_2D_pairplot_passages(experiments,randSeed):
    
    todosLosPasajes,todaLaInfo, todasS = totalExperimentsToPassages(experiments)
    perplex, tStrain = formatS_tSNE(todasS)
    tDTotal3D = apply_3D_tSNE(todosLosPasajes,randSeed,perplex)
    tSNE_Pairplot(tDTotal3D,todasS)
    return


def apply_tSNE_2D_scatterM_passages(experiments,randSeed):
    
    todosLosPasajes,todaLaInfo, todasS = totalExperimentsToPassages(experiments)
    perplex, tStrain = formatS_tSNE(todasS)
    tDTotal3D = apply_3D_tSNE(todosLosPasajes,randSeed,perplex)
    tSNE_PD_scatterM(tDTotal3D,tStrain)
    return


def apply_tSNE_3D_passages(experiments,randSeed):
    
    todosLosPasajes,todaLaInfo, todasS = totalExperimentsToPassages(experiments)
    perplex, tStrain = formatS_tSNE(todasS)
    tDTotal3D = apply_3D_tSNE(todosLosPasajes,randSeed,perplex)
    strains = tsne_MPL_Scatter_3D(tDTotal3D,tStrain)
    return strains


def apply_tSNE_2D_experiments(experiments,randSeed):
    
    todosLosExp, todasLasStr = totalExperimentsToPoints(experiments)
    perplex2, tStrain2 = formatS_tSNE(todasLasStr)
    tDTotalPoints2D = apply_2D_tSNE(todosLosExp,tStrain2,321, perplex2)
    strains = tsne_MPL_Scatter2D_exp(tDTotalPoints2D,tStrain2)
    return strains



    
    
    





from ReadData import readData
from GetEntropy import getEntropyExperiment, getEntropyExperimentAminos
from GetEntropySurface import getEntropySurface
from PlotEntropyPosition import plotEntropyPosition
from PCA import plotPCA
from EntropyInterpolation import interpolationPosition
import pandas as pd
import os
from PIL import Image
from PlotPopularDecay import plotPopularDecay
from PyQt5 import *
import sys

loaded_dfs = []
folder = ''
def commandExecute(argument):
    switcher ={
        '0': 'exit',
        '1': 'readDataExperiment',
        '2': 'loadDataExperiment',
        '3': 'getEntropy',
        '4': 'loadEntroy',
        '5': 'interpolateExperiment',
        '6': 'plotEntropySurface',
        '7': 'getPCA',
        '8': 'getCodonDecay',
        '9': 'plotEntropyPositionExperiment',
        '10': 'getLoadedExperiments'
    }
    ok = False
    while not ok:
        try:
            command = switcher[argument]
            ok = True
        except:
            argument =input('Escriba un comando valido: ')
    return eval(command + '()')

def main():
        print('#######################################################################')
        print('######################## VEV PROYECT ##################################')
        salir = False

        while not salir:
            print('1: Leer Nuevos Datos de Experimentos')
            print('2: Cargar Experimento ya procesado')
            print('3: Calcular Entropía')
            print('4: Cargar Entropía ya calculada')
            print('5: Interpolar Datos')
            print('6: Plotear Superficie de Entropía')
            print('7: Hacer PCA')
            print('8: Codon decay')
            print('9: Plotear Entropía por Posición')
            print('10: Ver Experimentos Cargados')
            print('0: Salir')
            command = input("Escriba el número de comando a ejecutar: ")
            salir=commandExecute(command)
            print("\n\n")

def exit():
    return True

def readDataExperiment():
    strain = input ("Escriba el nombre del experimento a leer: ")
    global folder
    if folder =='':
        folder = input ("Escriba la ubicación de los datos: ")
    repetition = input("Escriba el número de repetición: ")
    path = folder + '/' + strain + '_' + repetition+'/'
    print(path)
    parameters = [path,'Sample_p',strain,repetition,0,2482]
    try:
        df = readData(parameters)
        experimentName = strain + '_' + repetition + '.pkl'
        df.to_pickle(os.getcwd()+'/'+experimentName)
        loaded_dfs.append(experimentName)
        return False
    except:
        folder=''
        print("No se pudo cargar los datos, verifique los parametros envíados")
        return False


def loadDataExperiment():
    strain = input ("Escriba el nombre del experimento a cargar: ")
    folder = input ("Escriba la ubicación de los datos: ")
    repetition = input("Escriba el número de repetición: ")
    path = folder + '/' + strain + '_' + repetition + '/'
    parameters = [path,'Sample_p',strain,repetition,0,2482]

    experimentName = strain + '_' + repetition + '.pkl'
    if not experimentName in loaded_dfs:
        try:
            df = readData(parameters)
            df.to_pickle(os.getcwd() + '/' + experimentName)
            loaded_dfs.append(experimentName)
            return False
        except:
            print("No existe un experimento ya precargado con esos parametros")
            return False
    return False

def getEntropy():
    strain = input ("Escriba el nombre del experimento: ")
    repetition = input("Escriba el número de repetición: ")
    experimentName = strain + '_' + repetition + '.pkl'
    entropyName = strain + '_' + repetition + '_entropy.pkl'
    if not experimentName in loaded_dfs:
        print ("Primero debe cargar los datos del experimento")
        return False
    else:
        dfAux=pd.read_pickle(os.getcwd()+'/'+experimentName)
        df = getEntropyExperiment(os.getcwd(),strain,repetition,dfAux)
        df.to_pickle(os.getcwd() + '/' + entropyName)
        loaded_dfs.append(entropyName)
    return False

def loadEntroy():
    strain = input ("Escriba el nombre del experimento: ")
    repetition = input("Escriba el número de repetición: ")
    experimentName = strain + '_' + repetition + '.pkl'
    entropyName = strain + '_' + repetition + '_entropy.pkl'
    if not entropyName in loaded_dfs:
        try:
            dfAux = pd.read_pickle(os.getcwd()+'/'+experimentName)
            df = getEntropyExperiment(os.getcwd(), strain, repetition, dfAux)
            df.to_pickle(os.getcwd() + '/' + entropyName)
            loaded_dfs.append(entropyName)
            return False
        except:
            print('Primero debe calcular la entropia')
            return False

def interpolateExperiment():
    strain = input ("Escriba el nombre del experimento a interpolar: ")
    repetition = input("Escriba el número de repetición: ")
    pasaje = input("Escriba el pasaje: ")
    experimentName = strain + '_' + repetition+'_entropy.pkl'
    if not experimentName in loaded_dfs:
        print ("Primero debe cargar la entropia")
    else:
        try:
            df = pd.read_pickle(os.getcwd()+'/'+experimentName)
            df = interpolationPosition(pasaje,df)
            df.to_pickle(os.getcwd() + '/' + experimentName)
            return False
        except:
            print("El pasaje otorgado no es correcto")
            return False

    return False

def plotEntropySurface():
    strain = input ("Escriba el nombre del experimento a plotear: ")
    repetition = input("Escriba el número de repetición: ")
    experimentName = strain + '_' + repetition+'_entropy.pkl'
    if not experimentName in loaded_dfs:
        print ("Primero debe cargar la entropia")
    else:
        df = pd.read_pickle(os.getcwd()+'/'+experimentName)
        getEntropySurface(df)
    return False

def getPCA():
    finish = False
    dfAux = []
    position = input('Escriba la posición que desea resaltar: ')
    while not finish:
        strain = input ("Escriba el nombre del experimento: ")
        repetition = input("Escriba el número de repetición: ")
        experimentName = strain + '_' + repetition + '_entropy.pkl'
        if not experimentName in loaded_dfs:
            print ("La entropia no está calculada")
        else:
            dfAux.append(pd.read_pickle(os.getcwd()+'/'+experimentName))
            salir = input ("Escriba 'S' si desea cargar otro experimento, 'N' en caso contrario: ")
            if salir=='N' and len(dfAux) > 1:
                finish = True
            elif salir=='N' and len(dfAux) == 1:
                print('Debe cargar al menos 2 experimentos')
            elif salir!='N' and salir!='S':
                control = False
                while not control:
                    salir = input ("Escriba unicamente 'S' si desea cargar otro experimento, 'N' en caso contrario: ")
                    if salir == 'N':
                        finish = True
                        control = True
                    elif salir == 'S':
                        control = True
    plotPCA(dfAux,int(position))


def getCodonDecay():
    strain = input ("Escriba el nombre del experimento: ")
    repetition = input("Escriba el número de repetición: ")
    th = input("Escriba el valor del umbral: ")
    while float(th) >= 1:
        th = input("Escriba un valor de umbral menor que 1: ")

    experimentName = strain + '_' + repetition + '.pkl'
    if not experimentName in loaded_dfs:
        print ("Primero debe cargar los datos del experimento")
    else:
        df = pd.read_pickle(os.getcwd() + '/' + experimentName)
        plotPopularDecay(df, 1, 2000, float(th))
    return False

def plotEntropyPositionExperiment():
    strain = input ("Escriba el nombre del experimento: ")
    repetition = input("Escriba el número de repetición: ")
    position = input("Escriba la posición: ")
    experimentName = strain + '_' + repetition+'_entropy.pkl'
    if not experimentName in loaded_dfs:
        print ("Primero debe cargar la entropia del experimento")
    else:
        df = pd.read_pickle(os.getcwd()+'/'+experimentName)
        plotEntropyPosition(int(position), df, 'red')
        name = 'Entropy_' + str(position) + '.png'
        img = Image.open(name)
        titleAux = "Entropy for position "+position
        img.show(title=titleAux)
    return False



def getLoadedExperiments():
    listExp = ''
    for exp in loaded_dfs:
        listExp += exp + ' '
    print ('Los Experimentos cargados: '+listExp)
    return False





if __name__ == "__main__":

    main()



import numpy as np
import matplotlib
matplotlib.use('Qt5Agg')
from PyQt5 import QtWidgets
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from itertools import combinations
import pandas as pd



class FigureLVRV(FigureCanvas):

    def __init__(self,experiments, parent,nameExps,typelvrv=None,parameter=None,width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        FigureCanvas.__init__(self, fig)
        self.parent = parent
        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        self.experiments = experiments
        self.fig = fig
        self.array = []
        self.isImported = False
        if parameter == None:
            self.isImported =True

        self.name = ''
        self._flag = False
        self.points_marks = []
        self.type = typelvrv
        self.parameter = parameter
        self.nameExp = nameExps
        self.compute_initial_figure()


    def updatelvrv(self,dfs,nameExps=None,typeLVRV=None,param=None):
        self.array=[]
        self.experiments=dfs
        self.nameExp = nameExps
        self.type = typeLVRV
        self.parameter = param
        if param == None:
            self.isImported =True
        self.axes.cla()
        self.compute_initial_figure()


    def compute_initial_figure(self):
        if self.isImported:
            strains = self.experiments.Strain.unique()
            for s in strains:
                dfPlotAux = self.experiments[self.experiments.Strain.eq(s)]
                repetitions = dfPlotAux.Repetition.unique()
                for r in repetitions:
                    dfPlot = dfPlotAux[dfPlotAux.Repetition.eq(r)]
                    x = dfPlot.Entropy_Low
                    y = dfPlot.Entropy_High
                    aux = self.axes.scatter(x, y, alpha=0.5, label="Strain {s}, Repetition{r}".format(s=s,r=r))
                    self.array.append(aux)

        else:
            for df in self.experiments:
                info = (df.loc[0].Info).split(",")
                x = df.Entropy_Low
                y = df.Entropy_High
                aux =self.axes.scatter(x, y, alpha=0.5, label="Strain {s}, Repetition {r}".format(s=info[0], r=info[1]))
                self.array.append(aux)
        matplotlib.pyplot.grid()
        matplotlib.pyplot.legend(bbox_to_anchor=(1.1, 1.05))


        self.annot = self.axes.annotate("", xy=(0, 0), xytext=(20, 20), textcoords="offset points",
                            bbox=dict(boxstyle="round", fc="w"),
                            arrowprops=dict(arrowstyle="->"))

        self.annot.set_visible(False)
        self.fig.canvas.mpl_connect("motion_notify_event", self.hover)
        self.axes.legend()
        self.draw()


    def markRelatives(self,info):
        auxVal = []

        if self.isImported:
            auxPos = self.experiments[self.experiments.Position.eq(int(info[2]))]
            auxStrains = auxPos[auxPos.Strain.eq(info[0])]
            positions = auxStrains.values.tolist()
            if len(positions) > 0:
                for p in positions:

                    auxVal.append([p[6],p[7]])
        else:
            for dfPlot in self.experiments:
                positions = dfPlot.loc[dfPlot.Position.eq(info[2]) & dfPlot.Strain.eq(info[0])].values.tolist()
                if len(positions) > 0:
                    auxVal.append([positions[0][5],positions[0][6]])
        comb = combinations(auxVal, 2)
        for points in list(comb):
            point1 = points[0]
            point2 = points[1]
            x_values = [point1[0], point2[0]]
            y_values = [point1[1], point2[1]]
            self.axes.plot(x_values, y_values , color='red')


    def update_annot(self,ind,i):
        pos = self.array[i].get_offsets()[ind["ind"][0]]
        if self.isImported:
            dfPlot = self.experiments
        else:
            dfPlot = self.experiments[i]
        dinfo =dfPlot.loc[dfPlot.Entropy_Low.eq(pos[0]) & dfPlot.Entropy_High.eq(pos[1])]
        info = dinfo['Info'].values[0]
        auxInfo = info.split(',')
        self.markRelatives(auxInfo)
        self.annot.xy = pos
        pointInfo='Repetition: ' + auxInfo[1] + '\nPosition: ' + auxInfo[2]
        self.annot.set_text(pointInfo)
        self.annot.get_bbox_patch().set_alpha(0.4)


    def removeOldPoints(self):
        i = 0
        for _ in self.axes.lines:
            self.axes.lines[i].remove()
            i = i+1

    def hover(self,event):
        vis = self.annot.get_visible()
        if event.inaxes == self.axes:
            cont = False
            i = 0
            while not cont and i < len(self.array):
                cont, ind = self.array[i].contains(event)
                i = i +1
            if cont:
                self.removeOldPoints()
                self.update_annot(ind,(i-1))

                self.annot.set_visible(True)
                self.fig.canvas.draw_idle()

            else:
                if vis:
                    self.annot.set_visible(False)
                    self.fig.canvas.draw_idle()


    def exportCSV(self,filename):
        if self.isImported:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Unable to export an imported file')
            error_dialog.exec()
        else:
            i = 0
            for df in self.experiments:
                if i == 0:
                    dfR = df
                    i = 1
                else:
                    dfR = pd.concat([dfR,df])
            exps = ''
            i = 0
            j = 0
            dfR["type"] = df.apply(lambda row: self.type, axis=1)


            dfR.to_csv(filename)
            return self.nameExp,self.type,self.parameter




import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from sklearn.manifold import TSNE
from src.Format_DF import totalExperimentsToPassages, totalExperimentsToPositions, formatStrains


def tsnePassages(experiments, perp=None, lr=None, df_results=None):


    if ((perp==None) and (lr==None)):
        perps = [3, 6, 12, 15]
        learning_r = [10, 100, 300, 500]
    elif ((perp!=None) and (lr==None)):
        perps=perp
        learning_r = [10, 100, 300, 500]
    elif ((perp==None) and (lr!=None)):
        perps = [3, 6, 12, 15]
        learning_r=lr
    else:
        perps=perp
        learning_r=lr

    data, infos, strains = totalExperimentsToPassages(experiments)
    cant, df_Pas_Strains = formatStrains(strains)
    df_infos=pd.DataFrame(infos, columns=["Info"])


    data_Aux=pd.DataFrame(columns=["tsne_1", "tsne_2", "Strain", "StrainNumber", "Info", "Perp", "LR"])

    data_Tot=pd.DataFrame()


    for p in perps:
        for lr in learning_r:
            transformedData = TSNE(perplexity=p, learning_rate=lr, random_state=111).fit_transform(data)
            data_Aux["tsne_1"]=transformedData[:,0]
            data_Aux["tsne_2"]=transformedData[:,1]
            data_Aux["Strain"]=df_Pas_Strains["Strain"]
            data_Aux["StrainNumber"]=df_Pas_Strains["StrainNumber"]
            data_Aux["Info"]=df_infos["Info"]
            data_Aux.loc[:, "Perp"]=p
            data_Aux.loc[:, "LR"]=lr
            data_Aux.loc[:, "Plot"]=0
            data_Tot=data_Tot.append(data_Aux, ignore_index=True)



    if df_results is None:
        df_results=data_Tot
    else:
        df_results=df_results.append(data_Tot, ignore_index=True)

    return df_results

def tsnePositions(experiments, perp=None, lr=None, df_results=None):


    if ((perp==None) and (lr==None)):
        perps = [3, 12]
        learning_r = [10, 300]
    elif ((perp!=None) and (lr==None)):
        perps=perp
        learning_r = [10, 100, 300, 500]
    elif ((perp==None) and (lr!=None)):
        perps = [3, 6, 12, 15]
        learning_r=lr
    else:
        perps=perp
        learning_r=lr

    data, infos, strains = totalExperimentsToPositions(experiments)
    cant, df_Pas_Strains = formatStrains(strains)
    df_infos=pd.DataFrame(infos, columns=["Info"])


    data_Aux=pd.DataFrame(columns=["tsne_1", "tsne_2", "Strain", "StrainNumber", "Info", "Perp", "LR"])

    data_Tot=pd.DataFrame()


    for p in perps:
        for lr in learning_r:
            transformedData = TSNE(perplexity=p, learning_rate=lr, random_state=111).fit_transform(data)
            data_Aux["tsne_1"]=transformedData[:,0]
            data_Aux["tsne_2"]=transformedData[:,1]
            data_Aux["Strain"]=df_Pas_Strains["Strain"]
            data_Aux["StrainNumber"]=df_Pas_Strains["StrainNumber"]
            data_Aux["Info"]=df_infos["Info"]
            data_Aux.loc[:, "Perp"]=p
            data_Aux.loc[:, "LR"]=lr
            data_Aux.loc[:, "Plot"]=0
            data_Tot=data_Tot.append(data_Aux, ignore_index=True)



    if df_results is None:
        df_results=data_Tot
    else:
        df_results=df_results.append(data_Tot, ignore_index=True)

    return df_results


def plot_P_LR(data,perp,lr):

    data.loc[:,'Plot']=0
    data['Plot']=np.where(data.Perp.eq(perp) & data.LR.eq(lr),1,0)
    data.loc[(data['Perp'] == perp) & (data['LR'] == lr), 'Plot'] = 1

    dfPlot=data[data.Perp.eq(perp) & data.LR.eq(lr)]
    fig, ax = plt.subplots()

    strains=dfPlot.StrainNumber.unique()

    for s in strains:
        d=dfPlot.loc[dfPlot['StrainNumber'] == s]
        x=d.tsne_1
        y=d.tsne_2
        ax.scatter(x, y, label=s, alpha=0.3)
    ax.legend()
    plt.show()
#    p=dfPlot[['tsne_1','tsne_2', 'StrainNumber', 'Info']]
#    t2=dfPlot[['tsne_2']]
#    sn=dfPlot[['']]



from PyQt5.QtCore import QObject,QRunnable,pyqtSignal,pyqtSlot
from src.Filtrados import descomposicion_Poly_LV_RV,movingAverage_1D,movingAverage,descomposicion_PM_LV_RV

class lvrvSignals(QObject):

    finished = pyqtSignal(list)
    error = pyqtSignal()


class lvrv_poly(QRunnable):
    def __init__(self,parameters,grade,nameExps):
        super(lvrv_poly, self).__init__()
        self.experiments = parameters
        self.grade = grade
        self.nameExps = nameExps
        self.signals = lvrvSignals()



    @pyqtSlot()
    def run(self):
        try:
            l_LV_RV = []

            for df in self.experiments:
                descomposicion = descomposicion_Poly_LV_RV(df,self.grade)
                l_LV_RV.append(descomposicion)

        except Exception as e:

            print(e)

            self.signals.Error.emit()


        finally:
            self.signals.finished.emit([l_LV_RV,'Polynomial',self.nameExps,self.grade])  # Done


class lvrv_average(QRunnable):
    def __init__(self,parameters,window,nameExps):
        super(lvrv_average, self).__init__()
        self.experiments = parameters
        self.window = window
        self.nameExps = nameExps
        self.signals = lvrvSignals()


    @pyqtSlot()
    def run(self):
        try:
            l_LV_RV = []
            for df in self.experiments:
                descomposicion = descomposicion_PM_LV_RV(df,self.window)
                l_LV_RV.append(descomposicion)
        except Exception as e:
            print(e)
            self.signals.error.emit()
        finally:
            self.signals.finished.emit([l_LV_RV,'Moving Average',self.nameExps,self.window])  # Done


import numpy as np
import matplotlib
matplotlib.use('Qt5Agg')
from PyQt5 import QtWidgets
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure


dfPlot = ''

class FigureCD(FigureCanvas):

    def __init__(self,dfResult,df,th, parent,width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        FigureCanvas.__init__(self, fig)
        self.parent = parent
        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        self.dfResult = dfResult
        self.df = df
        self.th = th
        self.fig = fig
        self.auxInfo = []
        self.array = []
        self.compute_initial_figure()
        self._flag = False


    def updateCD(self,dfR,df,th):
        self.dfResult=dfR
        self.th = th
        self.df = df
        self.auxInfo = []
        self.array = []
        self.axes.cla()
        self.compute_initial_figure()


    def compute_initial_figure(self):
        passages = self.df['Passage'].unique()
        for pos in self.dfResult['Position']:
            maxF_Cod = self.dfResult.loc[self.dfResult['Position'] == pos].MaxFreqCodon.astype(str)
            freqEvol = self.df.loc[self.df['Position'] == pos]
            freqEvol = freqEvol[maxF_Cod.astype(str)]
            freqEvol = np.array(freqEvol)
            if (np.amin(freqEvol) < self.th):
                str_label = str(pos) + ' - ' + str(maxF_Cod.values.item())
                self.auxInfo.append(str_label)
                aux =self.axes.plot(passages, freqEvol)
                self.array.append(aux)
        self.axes.set_xlabel('Passages')
        self.axes.set_ylabel('Frequencies')

        self.annot = self.axes.annotate("", xy=(0, 0), xytext=(20, 20), textcoords="offset points",
                            bbox=dict(boxstyle="round", fc="w"),
                            arrowprops=dict(arrowstyle="->"))

        self.annot.set_visible(False)
        self.fig.canvas.mpl_connect("motion_notify_event", self.hover)
        self.axes.legend()
        self.draw()



    def update_annot(self,x,y,i):

        #pos = self.array[i][0].get_offsets()[ind["ind"][0]]
        self.annot.xy = (x,y)
        self.annot.set_text(self.auxInfo[i])
        self.annot.get_bbox_patch().set_alpha(0.4)



    def hover(self,event):
        vis = self.annot.get_visible()
        if event.inaxes == self.axes:
            cont = False
            i = 0
            while not cont and i < len(self.array):
                line = self.array[i][0]

                cont, ind = line.contains(event)
                i = i +1
            if cont:
                self.update_annot(event.xdata,event.ydata,(i-1))

                self.annot.set_visible(True)
                self.fig.canvas.draw_idle()

            else:
                if vis:
                    self.annot.set_visible(False)
                    self.fig.canvas.draw_idle()



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import LinearLocator, FormatStrFormatter


def getEntropySurface(dataFrame,strain,repetition):

    positions=np.array(dataFrame['Position'].astype(int))
    passages=np.array(dataFrame['Passage'].astype(int))
    entropy=np.array(dataFrame['Entropy'])


    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.grid(True)

    Y = positions/3

    X = passages


    Z = entropy


    surf = ax.plot_trisurf(X, Y, Z,  cmap='viridis')

    ax.zaxis.set_major_locator(LinearLocator(10))
    ax.zaxis.set_major_formatter(FormatStrFormatter('%.02f'))

# Add a color bar which maps values to colors.
    fig.colorbar(surf, shrink=0.5, aspect=5)



    ax.set_xlabel('Passages')
    ax.set_ylabel('Positions')
    ax.set_zlabel('Entropy')

    name = 'src/plot_images/entropySurface_' +strain+'_' +repetition+ '.png'
    plt.savefig(name, transparent=True)
    plt.close()


def getEntropySurface_Filtrados(df,strain,repetition):

    pos = df['Position'].unique()
    pos = list(map(int, pos))

    df_C=df.copy()
    df_C.drop(df_C.columns[[-3, -2, -1]], axis=1, inplace=True)

    passages = df_C.columns
    passages = list(map(int, passages))

    entropy = df_C.values

    X, Y = np.meshgrid(passages, pos)

    fig = plt.figure()
    ax = fig.gca(projection='3d')

    surf = ax.plot_surface(X, Y, entropy, rstride=1, cstride=1, cmap='viridis', linewidth=1, antialiased=True)
#    surf = ax.plot_surface(X, Y, entropy, linewidth=0, antialiased=False)

    fig.colorbar(surf, shrink=0.5, aspect=5)

#    plt.show()

    return fig



from PyQt5 import QtWidgets,uic, QtCore
from PyQt5.QtGui import QPixmap

path_buttons='src/buttons_and_background/'
class dError(QtWidgets.QDialog):
    def __init__(self, errorMessage,parent=None):
        super(dError, self).__init__()
        uic.loadUi('src/ui_files/dialogError.ui', self)
        self._flag = False
        self.pushButton.clicked.connect(self.closeWindow)
        self.pushButton.setStyleSheet("QPushButton{border-image:url("+path_buttons+"Ok.png);}")
        self.pushButton.setToolTip("Click to close window")
        self.errorText.setText(errorMessage)



    def closeWindow(self):
        self.close()



from PyQt5 import QtWidgets, uic, QtCore
from PyQt5.QtWidgets import QFileDialog, QLabel, QSlider
import sys
from src.ReadData import readData,keepValidPositions
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QPixmap, QPalette, QBrush, QImage, QIcon
from src.pcaClass import FigurePCA
from src.dialogPositions import valuesPositions
import pandas as pd
from src.dialogInterpolation import dInterpolation
from src.threadsOpenClass import LoadFile, LoadFolder
from src.threadsPlotClass import plotSurface,plotPositionEntropy,plotGumbel, codonDecay
from src.tsneClass import FigureTSNE
from src.tsneThreadProcess import tsneProcess,tsneProcessUpdate
from src.lvrvClass import FigureLVRV
from src.lvrvThread import lvrv_poly,lvrv_average
from src.codonDecayClass import FigureCD
from src.threadPCA import pcaProcess
from datetime import datetime
import logging
import pymongo
import socket


path_buttons='src/buttons_and_background/'
path_pkl = 'src/data_experiments/' #también a cambiar
path_grafics='src/plot_images'  #esto es algo a cambiar en todas las funciones que plotean
loaded_dfs = []

def dbconnect():
    client = pymongo.MongoClient("mongodb+srv://vevdb:MDp1atkxAv9R2KxZ@cluster0.lu8xi.gcp.mongodb.net/")
    db = client['VEV']
    return db



def logger(new,experiment,log):

    date_time = datetime.now().strftime("%d/%m/%Y, %H:%M:%S")

    if new:
        logFile = open(path_pkl+experiment,'w')
    else:
        logFile = open(path_pkl+experiment,'a')

    logAux = str(date_time) + ' ' + log
    logFile.write(logAux + '\n')
    logFile.close()
    logging.info(log)

def getExperiments(ne,nameExps):
    #299_1(774-7332)
    otherexperiments = ''
    for exp in nameExps:
        if ne != exp:
            experiment = exp.split('.')[0]
            otherexperiments =otherexperiments + experiment + ' '
    return otherexperiments


def initLoggin():
    date_time = datetime.now().strftime("%d-%m-%Y")
    logging.basicConfig(filename='logs/vev-'+date_time+'.log',format='%(asctime)s %(message)s', level=logging.INFO)
    logging.info('VEV Started')

class Interfaz(QtWidgets.QMainWindow):
    def __init__(self):
        super(Interfaz, self).__init__()
        uic.loadUi('src/ui_files/mainwindow.ui', self)
        self._flag = False
        self.threadpool = QtCore.QThreadPool()
        self.label_lvrv.hide()
        self.input_lvrv.hide()
        self.slidersTSNE = QtWidgets.QVBoxLayout(self.frame_sliders)
        self.arraySliders = []
        initLoggin()
        self.db = dbconnect()
        self.namePC = socket.gethostname()
        self.addUser()
        #call functions

        self.buttonCargar_experimento.clicked.connect(self.loadExperiment)
        self.buttonCargarPK.clicked.connect(self.loadExperimentPK)
        self.plotPositionButton.clicked.connect(self.plotPostionFunction)
        self.plotSurfaceButton.clicked.connect(self.plotSurfaceFunction)
        self.codonButton.clicked.connect(self.codonDecayFunction)
        self.PCAButton.clicked.connect(self.pcaFunction)
        self.tsneButton.clicked.connect(self.tsneFunctions)
        self.buttonInterpolation.clicked.connect(self.interpolateFunction)
        self.gumbelButton.clicked.connect(self.gumbelWidget)
        self.addPerpLRButton.clicked.connect(self.tsneAddPerpAndLR)
        self.mvAverage.toggled.connect(lambda: self.changeLabel(self.mvAverage))
        self.poly.toggled.connect(lambda: self.changeLabel(self.poly))
        self.lvrv_button.clicked.connect(self.lvrvFunction)

        #exports functions
        self.exportCSVTSNE.clicked.connect(self.exportTSNE)
        self.importCSVTSNE.clicked.connect(self.importTSNE)
        self.exportCSVLVRV.clicked.connect(self.exportLVRV)
        self.ImportCSVLVRV.clicked.connect(self.importLVRV)
        self.ExportCSVPCA.clicked.connect(self.exportPCA)
        self.ImportCSVPCA.clicked.connect(self.importPCA)

        # Move stacked widgets

        self.stackedWidget.setCurrentIndex(0)  # main
        self.buttonplotposition.clicked.connect(self.entropyPosition)
        self.buttonEntropySurface.clicked.connect(self.entropySurface)
        self.buttonCodonDecay.clicked.connect(self.codonDecayWidget)
        self.buttonPCAFunction.clicked.connect(self.pcaWidget)
        self.buttonTsne.clicked.connect(self.tsne)
        self.plotGumbelButton.clicked.connect(self.gumbelFunction)
        self.buttonLVRV.clicked.connect(self.LVRV)

        # style
        self.setMinimumSize(1000, 568)
        self.setWindowIcon(QIcon(path_buttons+'ImageICON.jpg'))
        self.buttonCargar_experimento.setStyleSheet("QPushButton{border-image:url("+path_buttons+"Load Folder.png);}")
        self.buttonCargar_experimento.setToolTip("Click this button to open experiment folder")
        self.buttonCargarPK.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "Load File.png);}")
        self.buttonCargarPK.setToolTip("Click this button to open experiment file")
        self.buttonLVRV.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "LVRV.png);}")
        self.buttonLVRV.setToolTip("Plot Leading Variations and Random Variations")
        self.buttonplotposition.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "Plot Entropy x Position.png);}")
        self.buttonplotposition.setToolTip("Plot Entropy times position")
        self.buttonEntropySurface.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "Plot Entropy Surface.png);}")
        self.buttonEntropySurface.setToolTip("Plot Entropy Surface of an experiment")
        self.buttonCodonDecay.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "Codon Decay.png);}")
        self.buttonCodonDecay.setToolTip("Plot Codon Decay of an experiment")
        self.buttonPCAFunction.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "PCA.png);}")
        self.buttonPCAFunction.setToolTip("Plot PCA")
        self.gumbelButton.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "Gumbel.png);}")
        self.gumbelButton.setToolTip("Plot Gumbel distribution")
        self.buttonTsne.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "t-SNE.png);}")
        self.buttonTsne.setToolTip("Enter to t-SNE plot options")
        self.buttonInterpolation.setStyleSheet("QPushButton{border-image:url(" + path_buttons + "Interpolation.png);}")
        self.buttonInterpolation.setToolTip("Interpolate experiment data")

        #TABLE VIEW MODELS##
        self.model = QStandardItemModel()
        self.model.setHorizontalHeaderLabels(['Experiment '])
        self.tableView_PlotPostion.setModel(self.model)
        self.tableView_PlotPostion.horizontalHeader().setSectionResizeMode(1)
        self.tableTSNE.setModel(self.model)
        self.tableTSNE.horizontalHeader().setSectionResizeMode(1)
        self.tablePCA.setModel(self.model)
        self.tablePCA.horizontalHeader().setSectionResizeMode(1)
        self.tableViewCodon.setModel(self.model)
        self.tableViewCodon.horizontalHeader().setSectionResizeMode(1)
        self.tableViewEntropySurface.setModel(self.model)
        self.tableViewEntropySurface.horizontalHeader().setSectionResizeMode(1)
        self.tableViewGumbel.setModel(self.model)
        self.tableViewGumbel.horizontalHeader().setSectionResizeMode(1)
        self.tableLVRV.setModel(self.model)
        self.tableLVRV.horizontalHeader().setSectionResizeMode(1)
        #Sliders
        self.perplexitySlider.setMaximum(3)
        self.perplexitySlider.setMinimum(0)
        self.lrSlider.setMaximum(3)
        self.lrSlider.setMinimum(0)

        self.perplexitySlider.valueChanged.connect(self.updateTSNE)
        self.lrSlider.valueChanged.connect(self.updateTSNE)
        #FIGURES
        self.pca=None
        self.tsne=None
        self.lvrv=None
        self.condonDecay=None
        #AUXILIAR
        self.perAux=[3,6,12,15]
        self.lrAux=[10,100,300,500]

    def scale_image(self,size):
        imageBack = QImage(path_buttons+'background.png')
        #imageBack= imageBack.scaled(size,QtCore.Qt.KeepAspectRatio)
        palette = QPalette()
        palette.setBrush(QPalette.Window, QBrush(imageBack))
        self.setPalette(palette)

    def resizeEvent(self, e):
        if not self._flag:
            self._flag = True
            self.scale_image(e.size())
            QtCore.QTimer.singleShot(30, lambda: setattr(self, "_flag", False))
        super().resizeEvent(e)

#CHANGE LABEL
    def changeLabel(self, b): #b is the radio button
        self.label_lvrv.show()
        self.input_lvrv.show()
        if b.text() == "Moving Average":
            if b.isChecked() == True:
                self.label_lvrv.setText("Select Average Window:")
            else:
                self.label_lvrv.setText("Select Polynomial Grade:")

        else:
            if b.isChecked() == True:
                self.label_lvrv.setText("Select Polynomial Grade:")
            else:
                self.label_lvrv.setText("Select Average Window:")
#Load FILES AND FOLDERS

    def loadExperiment(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        fileName = QFileDialog.getExistingDirectory(self, 'Select a directory', options=options)
        if fileName != "":
            vp = valuesPositions()
            vp.exec()
            fvp = vp.getFVP()
            lvp = vp.getLVP()
            thread = LoadFolder(fileName,fvp,lvp)
            thread.signals.error.connect(self.openError)
            thread.signals.finished.connect(self.openOK)
            self.threadpool.start(thread)

    def loadExperimentPK(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        (fileName, aux) = QFileDialog.getOpenFileName(self, caption='Select a .parquet File', filter='*.parquet',options=options)
        if fileName != "":
            thread = LoadFile(fileName)
            thread.signals.error.connect(self.openError)
            thread.signals.finished.connect(self.openOK)
            self.threadpool.start(thread)

# Complete or Errors with open

    def openOK(self,parameters):
        print (parameters)
        strain = parameters[0]
        repetition = parameters[1]
        fvp = parameters[2]
        lvp = parameters[3]
        control = parameters[4]
        if (strain, repetition, fvp, lvp) in loaded_dfs:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('This experiment is already loaded')
            error_dialog.exec()
        else:
            loaded_dfs.append((strain, repetition, fvp, lvp))
            experimentName = "Strain: " + strain + " " + "Repetition: " + repetition
            row = (QStandardItem(experimentName))
            self.model.appendRow(row)
            self.refreshFunction()
            if control:
                nameLog=strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
                log = 'Experiment was loaded and entropy was calculated with First Value Position(FVP) = '+str(fvp)+' and Last Value Position(LVP) = '+str(lvp)
                logger(True,nameLog,log)
                de = QtWidgets.QMessageBox()
                de.setText('Folder: ' + experimentName + ' loaded')
                de.exec()
            else:

                logging.info(' Experiment already calculated was load: '+strain+'_'+repetition+'('+str(fvp)+'-'+str(lvp)+')')


    def openError(self,error):
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage(str(error))
        error_dialog.exec()
        logging.info('An error has occurred when an experiment was loaded')


    def refreshFunction(self):
        self.tablePCA.update()
        self.tableViewEntropySurface.update()
        self.tableView_PlotPostion.update()
        self.tableViewCodon.update()
        self.tableTSNE.update()
        self.tableViewGumbel.update()
        self.tableLVRV.update()

    def interpolateFunction(self): #ver posibilidad de hilo
        di = dInterpolation(loaded_dfs)
        di.exec()
        executed, typeInterpolation,strain,repetition,fvp,lvp,passage = di.getInfo()
        if executed:
            nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
            log = typeInterpolation + ' interpolation was made for passage number ' +passage
            logger(False, nameLog, log)

    # PLOTS AND LOADS FUNCTIONS

    def tsneAddPerpAndLR(self):
        try:

            perp = int(self.newPerplexity.toPlainText())
            lr   = int(self.newLR.toPlainText())

            dfs,data,nameExps = self.tsne.getData()
            print ("info return from getdata: "+str(nameExps))
            auxType= self.tsne.getType()
            if auxType == 'Passages':
                thread = tsneProcessUpdate(dfs,perp,lr,data,"Passages",nameExps)
            else:
                thread = tsneProcessUpdate(dfs,perp,lr,data,"Positions",nameExps)
            thread.signals.error.connect(self.errorTSNEProcess)
            thread.signals.finished.connect(self.okProcessTSNEUpdate)
            self.threadpool.start(thread)

        except Exception as e:
            print(e)
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please enter integer values')
            error_dialog.exec()
    def okProcessTSNEUpdate(self,parameters):

        data = parameters[0]
        perp = parameters[1]
        lr = parameters[2]
        typeTSNE = parameters[3]
        nameExps = parameters[4]

        self.tsne.updateData(data,perp,lr,nameExps)
        for ne in nameExps:
            otherexperiments = getExperiments(ne,nameExps)
            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]
            log = typeTSNE +' t-SNE executed with perplexity = '+str(perp) +' and learning rate = '+ str(lr) + ' with this experiments: '+otherexperiments
            #logger(False, ne, log)
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp, 't-SNE '+typeTSNE+' with perp='+str(perp)+' and lr='+str(lr), otherexperiments, '')

    def clearLayout(self, layout):
        self.arraySliders=[]
        if layout:
            while layout.count():
                item = layout.takeAt(0)
                widget = item.widget()
                if widget:
                    widget.deleteLater()
                else:
                    self.clearLayout(item.layout())
                layout.removeItem(item)

    def addSlidersTSNE(self,strains):
        self.clearLayout(self.slidersTSNE)

        for strain in strains:
            label = QLabel()
            label.setText(str(strain))
            self.slidersTSNE.addWidget(label)
            slid = QSlider(QtCore.Qt.Horizontal)
            slid.setMaximum(1)
            slid.setMinimum(0)
            self.slidersTSNE.addWidget(slid)
            slid.setValue(0)
            slid.valueChanged.connect(self.updateStrainsTSNE)
            self.arraySliders.append([slid,strain])

    def tsneFunctions(self):
        dfsAux = []
        dfs = []
        strains = []
        nameExperiments= []
        indices = self.tableTSNE.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()


        elif (self.rb_passages.isChecked()==False and self.rb_positions.isChecked()==False):
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select method type')
            error_dialog.exec()

        else:
            try:
                fvpMayor = 0
                lvpMenor = 0
                for index in sorted(indices):
                    experimentIndex = (index.row())
                    df = loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    if strain not in strains:
                        strains.append(strain)
                    repetition = str(df[1])
                    fvp = df[2]
                    lvp = df[3]
                    if fvpMayor == 0 and lvpMenor == 0:
                        fvpMayor = fvp
                        lvpMenor = lvp
                    else:
                        if fvp > fvpMayor:
                            fvpMayor = fvp
                        if lvp < lvpMenor:
                            lvpMenor = lvp
                    experimentNametxt = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
                    experimentName = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')_entropy.parquet'
                    dfAux = pd.read_parquet(path_pkl + experimentName)
                    dfsAux.append(dfAux)
                    nameExperiments.append(experimentNametxt)
                for dataFrame in dfsAux:
                    df = keepValidPositions(dataFrame, fvpMayor, lvpMenor)
                    dfs.append(df)

                self.addSlidersTSNE(strains)
                if self.rb_passages.isChecked()==True:
                    auxType = 'Passages'
                else:
                    auxType = 'Positions'
                thread = tsneProcess(dfs,auxType,nameExperiments)
                thread.signals.error.connect(self.errorTSNEProcess)
                thread.signals.finished.connect(self.okProcessTSNE)
                self.threadpool.start(thread)
            except Exception as e:
                print (e)
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error')
                error_dialog.exec()

    def errorTSNEProcess(self):
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated t-SNE')
        error_dialog.exec()
        logging.info('An error has occurred when calculated t-SNE')

    def okProcessTSNE(self,parameters):
        dfs = parameters[0]
        data = parameters[1]
        tsneType = parameters[2]
        nameExp = parameters[3]
        if self.tsne == None:
            boxAux = QtWidgets.QVBoxLayout(self.tsneFrame)
            self.tsne = FigureTSNE(dfs,data,tsneType,nameExp, self.tsneFrame, width=6, height=5, dpi=100)
            boxAux.addWidget(self.tsne)
        else:
            self.perplexitySlider.valueChanged.disconnect()
            self.lrSlider.valueChanged.disconnect()
            self.tsne.newtSNE(dfs,data,tsneType,nameExp)
            self.perplexitySlider.setValue(0)
            self.lrSlider.setValue(0)
            self.perplexity.setText(str(3))
            self.lr.setText(str(10))
            self.perplexitySlider.valueChanged.connect(self.updateTSNE)
            self.lrSlider.valueChanged.connect(self.updateTSNE)
        if tsneType == 'Passages':
            self.perplexitySlider.setMaximum(3)
            self.lrSlider.setMaximum(3)
            self.perAux = [3, 6, 12, 15]
            self.lrAux = [10, 100, 300, 500]
        else:
            self.perplexitySlider.setMaximum(1)
            self.lrSlider.setMaximum(1)
            self.perAux = [3, 12]
            self.lrAux = [10, 300]
        for ne in nameExp:
            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]
            otherexperiments = getExperiments(ne,nameExp)
            log = tsneType+' t-SNE executed with this experiments: '+otherexperiments
            #logger(False, ne, log)
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp, 't-SNE '+tsneType, otherexperiments, '')

    def updateTSNE(self):
        perplexity = self.perAux[self.perplexitySlider.value()]
        lr = self.lrAux[self.lrSlider.value()]
        self.perplexity.setText(str(perplexity))
        self.lr.setText(str(lr))
        if self.tsne==None:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please before change this, plot t-SNE method')
            error_dialog.exec()
            self.perplexitySlider.valueChanged.disconnect()
            self.lrSlider.valueChanged.disconnect()
            self.perplexitySlider.setValue(0)
            self.lrSlider.setValue(0)
            self.perplexity.setText(str(3))
            self.lr.setText(str(10))
            self.perplexitySlider.valueChanged.connect(self.updateTSNE)
            self.lrSlider.valueChanged.connect(self.updateTSNE)
        else:
            self.tsne.updateParameters(perplexity,lr)

    def getChanges(self):
        changes = []
        for [slider, strain] in self.arraySliders:
            if slider.value() == 1:
                changes.append(strain)
        return changes

    def updateStrainsTSNE(self):
        if self.tsne==None:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please before change this, plot t-SNE method')
            error_dialog.exec()
        else:
            changes =self.getChanges()
            self.tsne.onOffStrains(changes)

    def pcaFunction(self):  #PASAR PCA A HILOS
        dfsAux = []
        dfs = []
        nameExps = []
        indices = self.tablePCA.selectionModel().selectedRows()

        if len(indices) < 2:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least two Experiments')
            error_dialog.exec()

        else:
            try:
                fvpMayor = 0
                lvpMenor = 0
                for index in sorted(indices):
                    experimentIndex = (index.row())
                    df = loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    repetition = str(df[1])
                    fvp =df[2]
                    lvp =df[3]
                    if fvpMayor == 0 and lvpMenor ==0:
                        fvpMayor = fvp
                        lvpMenor = lvp
                    else:
                        if fvp > fvpMayor:
                            fvpMayor = fvp
                        if lvp < lvpMenor:
                            lvpMenor = lvp
                    experimentNametxt = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
                    experimentName = strain + '_' + repetition +'('+str(fvp)+'-'+str(lvp)+ ')_entropy.parquet'
                    dfAux = pd.read_parquet(path_pkl + experimentName)
                    dfsAux.append(dfAux)
                    nameExps.append(experimentNametxt)
                for dataFrame in dfsAux:
                    df =keepValidPositions(dataFrame,fvpMayor,lvpMenor)
                    dfs.append(df)

                thread = pcaProcess(dfs,nameExps)
                thread.signals.error.connect(self.errorPCA)
                thread.signals.finished.connect(self.pcaOK)
                self.threadpool.start(thread)
            except Exception as e:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage(str(e))
                error_dialog.exec()


    def pcaOK(self,parameters):
        # def __init__(self, dfPuntosPCA, infoDF, pcaVector, nameExps,dfs, parent, width=5, height=4, dpi=100):
        #[dfPuntosPCA,pcaVector,infoDF, self.nameExp,self.data]
        dfPuntosPCA = parameters[0]
        pcaVector = parameters[1]
        infoDF = parameters[2]
        nameExps = parameters[3]
        dfs = parameters[4]
        if self.pca == None:
            boxAux = QtWidgets.QVBoxLayout(self.widget_image)
            self.pca = FigurePCA(dfPuntosPCA,pcaVector,infoDF,nameExps,dfs, self.widget_image, width=6, height=5, dpi=100)
            boxAux.addWidget(self.pca)
        else:
            self.pca.updatePCA(dfPuntosPCA,pcaVector,infoDF,nameExps,dfs)
        for ne in nameExps:
            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]
            otherexperiments = getExperiments(ne, nameExps)
            log = 'PCA executed with this experiments ' + otherexperiments
            #logger(False, ne, log)
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp, 'PCA', otherexperiments, '')

    def errorPCA(self):
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated PCA')
        error_dialog.exec()
        logging.info('An error has occurred when calculated PCA')

    def codonDecayFunction(self):
        indices = self.tableViewCodon.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:
            try:
                th = self.threshold.toPlainText()
                fth = float(th)
                for index in sorted(indices):
                    experimentIndex = (index.row())
                df = loaded_dfs[experimentIndex]
                strain = str(df[0])
                repetition = str(df[1])
                fvp = str(df[2])
                lvp = str(df[3])
                thread = codonDecay(strain, repetition,fth,fvp,lvp)
                thread.signals.error.connect(self.errorPlot)
                thread.signals.finished.connect(self.codonDecayOK)
                self.threadpool.start(thread)
            except Exception as e:
                if e.__class__ == FileNotFoundError:
                    error_dialog = QtWidgets.QErrorMessage()
                    error_dialog.showMessage('Error: Please enter the experiment pkl with frequencies')
                    error_dialog.exec()
                else:
                    error_dialog = QtWidgets.QErrorMessage()
                    error_dialog.showMessage('Error: Please enter an float threshold')
                    error_dialog.exec()

    def codonDecayOK(self,parameters):
        dfResult = parameters[0]
        df = parameters[1]
        th = parameters[2]
        strain = parameters[3]
        repetition = parameters[4]
        fvp = parameters[5]
        lvp = parameters[6]
        if self.condonDecay == None:
            boxAux = QtWidgets.QVBoxLayout(self.codonDecayFrame)
            self.condonDecay = FigureCD(dfResult,df,th, self.codonDecayFrame, width=6, height=5, dpi=100)
            boxAux.addWidget(self.condonDecay)
        else:
            self.condonDecay.updateCD(dfResult,df,th)
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
        log = 'Codon Decay was plotted with threshold '+str(th)
        # logger(False, ne, log)
        logging.info(log)
        self.addEntrydb(strain, repetition, fvp, lvp, 'Codon Decay', th, '')


    def plotSurfaceFunction(self):
        indices = self.tableViewEntropySurface.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:
            for index in sorted(indices):
                experimentIndex = (index.row())
            df = loaded_dfs[experimentIndex]
            strain = str(df[0])
            repetition = str(df[1])
            fvp = str(df[2])
            lvp = str(df[3])
            thread = plotSurface(strain,repetition, fvp, lvp)
            thread.signals.error.connect(self.errorPlot)
            thread.signals.finished.connect(self.plotSurfaceOK)
            self.threadpool.start(thread)


    def plotSurfaceOK(self,parameters):
            strain = parameters[0]
            repetition = parameters[1]
            fvp = parameters[2]
            lvp = parameters[3]
            name = path_grafics + '/entropySurface_' + strain + '_' + repetition + '.png'
            pixmap = QPixmap(name)
            #pixmap = pixmap.scaled(700, 700, QtCore.Qt.KeepAspectRatio)
            self.imageSurface.setPixmap(pixmap)
            nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
            log = 'Entropy surface is plotted and saved'
            # logger(False, ne, log)
            logging.info(log)
            self.addEntrydb(strain, repetition, fvp, lvp, 'Plot Surface Entropy', '', name)




    def plotPostionFunction(self):
        indices = self.tableView_PlotPostion.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:
            position = self.textEditPosition.toPlainText()
            for index in sorted(indices):
                experimentIndex = (index.row())
            df = loaded_dfs[experimentIndex]
            strain = str(df[0])
            repetition = str(df[1])
            fvp=str(df[2])
            lvp=str(df[3])
            thread = plotPositionEntropy(strain,repetition,position, fvp, lvp)
            thread.signals.error.connect(self.errorPlot)
            thread.signals.finished.connect(self.plotEntropyPositionOK)
            self.threadpool.start(thread)

    def plotEntropyPositionOK(self, parameters):
        strain = parameters[0]
        repetition = parameters[1]
        position = parameters[2]
        fvp = parameters[3]
        lvp = parameters[4]
        name = path_grafics + '/Entropy_' + str(position) + '_' + strain + '_' + repetition + '.png'
        pixmap = QPixmap(name)
        self.plotPositionImage.setPixmap(pixmap)
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
        log = 'Entropy is plotted and saved for position '+str(position)
        logger(False, nameLog, log)
        self.addEntrydb(strain,repetition,fvp,lvp,'Entropy times Position',position,name)

    def gumbelFunction(self):
        indices = self.tableViewGumbel.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()
        elif len(indices) > 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select only one Experiment')
            error_dialog.exec()
        else:
            for index in sorted(indices):
                experimentIndex = (index.row())
            df = loaded_dfs[experimentIndex]
            strain = str(df[0])
            repetition = str(df[1])
            fvp = str(df[2])
            lvp = str(df[3])
            thread = plotGumbel(strain, repetition,fvp,lvp)
            thread.signals.error.connect(self.errorPlot)
            thread.signals.finished.connect(self.plotGumbelOK)
            self.threadpool.start(thread)



    def plotGumbelOK(self, parameters):
        strain = parameters[0]
        repetition = parameters[1]
        fvp=parameters[2]
        lvp = parameters[3]
        name = path_grafics + '/Gumbel'+ '_' + strain + '_' + repetition + '.png'
        pixmap = QPixmap(name)
        self.imageGumbel.setPixmap(pixmap)
        nameLog = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
        log = 'Gumbel is plotted and saved'
        # logger(False, ne, log)
        logging.info(log)
        self.addEntrydb(strain, repetition, fvp, lvp, 'Gumbel', '', name)

    def errorPlot(self, e):
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage(str(e))
        error_dialog.exec()
        logging.info('An error has occurred when experiment was plotted')


    def lvrvFunction(self):
        dfsAux = []
        dfs = []
        nameExp = []
        indices = self.tableLVRV.selectionModel().selectedRows()
        if len(indices) < 1:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select at least one Experiment')
            error_dialog.exec()


        elif (self.mvAverage.isChecked()==False and self.poly.isChecked()==False ):
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please select method type')
            error_dialog.exec()

        else:
            try:
                input = self.input_lvrv.toPlainText()
                intInput = int(input)
                fvpMayor = 0
                lvpMenor = 0
                for index in sorted(indices):
                    experimentIndex = (index.row())
                    df = loaded_dfs[experimentIndex]
                    strain = str(df[0])
                    repetition = str(df[1])
                    fvp = df[2]
                    lvp = df[3]
                    if fvpMayor == 0 and lvpMenor == 0:
                        fvpMayor = fvp
                        lvpMenor = lvp
                    else:
                        if fvp > fvpMayor:
                            fvpMayor = fvp
                        if lvp < lvpMenor:
                            lvpMenor = lvp
                    experimentNametxt = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ').txt'
                    experimentName = strain + '_' + repetition + '(' + str(fvp) + '-' + str(lvp) + ')_entropy.parquet'
                    dfAux = pd.read_parquet(path_pkl + experimentName)
                    dfsAux.append(dfAux)
                    nameExp.append(experimentNametxt)
                for dataFrame in dfsAux:
                    df = keepValidPositions(dataFrame, fvpMayor, lvpMenor)
                    dfs.append(df)
                if self.mvAverage.isChecked()==True:
                    thread = lvrv_average(dfs,intInput,nameExp)
                    thread.signals.finished.connect(self.lvrvOK)
                    thread.signals.error.connect(self.lvrvError)
                    self.threadpool.start(thread)
                else:
                    thread = lvrv_poly(dfs,intInput,nameExp)
                    thread.signals.finished.connect(self.lvrvOK)
                    thread.signals.error.connect(self.lvrvError)
                    self.threadpool.start(thread)
            except Exception as e:
                print (e)
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please enter an integer')
                error_dialog.exec()

    def lvrvError(self):
        error_dialog = QtWidgets.QErrorMessage()
        error_dialog.showMessage('Error: An error has occurred when calculated LV-RV')
        error_dialog.exec()
        logging.info('An error has occurred when calculated LV-RV')

    def lvrvOK(self,parameters):
        dfs = parameters[0]
        typeLVRV = parameters[1]
        nameExps = parameters[2]
        param = parameters[3]
        if self.lvrv == None:
            boxAux = QtWidgets.QVBoxLayout(self.lvrv_frame)
            self.lvrv = FigureLVRV(dfs, self.lvrv_frame,nameExps,typeLVRV,param, width=6, height=5, dpi=100)
            boxAux.addWidget(self.lvrv)
        else:
            self.lvrv.updatelvrv(dfs,nameExps,typeLVRV,param)
        for ne in nameExps:

            auxStrain = ne.split("_")
            strain = auxStrain[0]
            auxRep = auxStrain[1].split("(")
            repetition = auxRep[0]
            auxfvp = auxRep[1].split("-")
            fvp = auxfvp[0]
            lvp = auxfvp[1].split(")")[0]

            otherexperiments = getExperiments(ne,nameExps)

            log = typeLVRV +' LV-RV was plotted with parameter '+str(param)+' with this experiments: '+otherexperiments
            #logger(False, ne, log)
            logging.info(log)
            self.addEntrydb(strain,repetition,fvp,lvp,typeLVRV+' LV-RV',otherexperiments+' with parameter = '+str(param),'')

    # functions for move stackedWidget
    def entropyPosition(self):
        self.stackedWidget.setCurrentIndex(1)  # plotEntropyPosition
    def entropySurface(self):
        self.stackedWidget.setCurrentIndex(2)
    def codonDecayWidget(self):
        self.stackedWidget.setCurrentIndex(4)
    def tsne(self):
        self.stackedWidget.setCurrentIndex(5)
    def pcaWidget(self):
        self.stackedWidget.setCurrentIndex(3)
    def gumbelWidget(self):
        self.stackedWidget.setCurrentIndex(6)

    def LVRV(self):
        self.stackedWidget.setCurrentIndex(7)

    #functions to export

    def exportTSNE(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        fileName, _ = QFileDialog.getSaveFileName(self, caption='Save experiment', filter='*.csv',
                                                      options=options)
        if fileName !='':
            if '.csv' not in fileName:
                fileName = fileName.replace('.','') + '.csv'
            if self.tsne == None:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please Plot t-SNE before export')
                error_dialog.exec()
            else:

                nameExp,normal,lr,perp,tsneType = self.tsne.exportCSV(fileName)
                for ne in nameExp:
                    auxStrain = ne.split("_")
                    strain = auxStrain[0]
                    auxRep = auxStrain[1].split("(")
                    repetition = auxRep[0]
                    auxfvp = auxRep[1].split("-")
                    fvp = auxfvp[0]
                    lvp = auxfvp[1].split(")")[0]
                    otherexperiments = getExperiments(ne, nameExp)
                    if normal:
                        log = tsneType + ' t-SNE executed with this experiments: ' + otherexperiments

                        logging.info(log)
                        self.addEntrydb(strain, repetition, fvp, lvp, 't-SNE '+tsneType, otherexperiments, fileName)
                    else:
                        log = tsneType + ' t-SNE executed with perplexity = ' + str(perp) + ' and learning rate = ' + str(lr) + ' with this experiments: ' + otherexperiments
                        logging.info(log)
                        self.addEntrydb(strain, repetition, fvp, lvp,'t-SNE ' + tsneType + ' with perp=' + str(perp) + ' and lr=' + str(lr), otherexperiments, fileName)


    def importTSNE(self):
        try:
            nameExps = []
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            options |= QFileDialog.DontUseCustomDirectoryIcons
            (fileName, aux) = QFileDialog.getOpenFileName(self, caption='Select a .csv File', filter='*.csv',options=options)
            #/home/rodrigo/Proyecto/vev/src/exports/tSNE-exps-WT_1(774-7332)-299_1(774-7332).csv
            if fileName != "":
                df = pd.read_csv(fileName)
                strains = df.Strain.unique()
                self.addSlidersTSNE(strains)
                columns = df.columns.unique()
                if 'Position' in columns:
                    tsneType = 'Positions'
                else:
                    tsneType = 'Passages'
                perps = df.Perp.unique()
                lrs = df.LR.unique()

                if len(perps) == 1 and len(lrs)==1:
                    perp = perps[0]
                    lr = lrs[0]
                else:
                    lr= None
                    perp = None


                if self.tsne == None:
                    boxAux = QtWidgets.QVBoxLayout(self.tsneFrame)
                    self.tsne = FigureTSNE(None, df, tsneType, nameExps, self.tsneFrame,lr,perp, width=6, height=5, dpi=100)
                    boxAux.addWidget(self.tsne)
                else:
                    #def newtSNE(self,dataframes,data, tsneType,nameExp,lr=None,perp=None):
                    self.tsne.newtSNE(None, df, tsneType, nameExps,lr,perp)
                self.perplexitySlider.valueChanged.disconnect()
                self.lrSlider.valueChanged.disconnect()
                if tsneType == 'Passages':
                    self.perplexitySlider.setValue(0)
                    self.lrSlider.setValue(0)
                    self.perplexitySlider.setMaximum(3)
                    self.lrSlider.setMaximum(3)
                    self.perAux = [3, 6, 12, 15]
                    self.lrAux = [10, 100, 300, 500]
                else:

                    self.perplexitySlider.setMaximum(1)
                    self.lrSlider.setMaximum(1)
                    self.perAux = [3,12]
                    self.lrAux = [10,300]

                    self.perplexitySlider.setValue(0)
                    self.lrSlider.setValue(0)
                    #self.perplexity.setText(str(3))
                    #self.lr.setText(str(10))

                self.perplexitySlider.valueChanged.connect(self.updateTSNE)
                self.lrSlider.valueChanged.connect(self.updateTSNE)
        except Exception as e:
            print(e)
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please import a valid t-SNE experiment CSV')
            error_dialog.exec()


    def exportLVRV(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog
        options |= QFileDialog.DontUseCustomDirectoryIcons
        fileName, _ = QFileDialog.getSaveFileName(self, caption='Save experiment', filter='*.csv',
                                                      options=options)
        if fileName !='':
            if '.csv' not in fileName:
                fileName = fileName.replace('.','') + '.csv'
            if self.lvrv == None:
                error_dialog = QtWidgets.QErrorMessage()
                error_dialog.showMessage('Error: Please Plot LV-RV before export')
                error_dialog.exec()
            else:
                nameExps,typeLVRV,param = self.lvrv.exportCSV(fileName)
                for ne in nameExps:
                    auxStrain = ne.split("_")
                    strain = auxStrain[0]
                    auxRep = auxStrain[1].split("(")
                    repetition = auxRep[0]
                    auxfvp = auxRep[1].split("-")
                    fvp = auxfvp[0]
                    lvp = auxfvp[1].split(")")[0]

                    otherexperiments = getExperiments(ne, nameExps)

                    log = typeLVRV + ' LV-RV was plotted with parameter ' + str(
                        param) + ' with this experiments: ' + otherexperiments
                    # logger(False, ne, log)
                    logging.info(log)
                    self.addEntrydb(strain, repetition, fvp, lvp, typeLVRV + ' LV-RV',otherexperiments + ' with parameter = ' + str(param), fileName)

    def importLVRV(self):
        try:

            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            options |= QFileDialog.DontUseCustomDirectoryIcons
            (fileName, aux) = QFileDialog.getOpenFileName(self, caption='Select a .csv File', filter='*.csv',
                                                          options=options)

            if fileName != "":
                df = pd.read_csv(fileName)
                lvrvType = df.type.unique()[0]

                if self.lvrv == None:
                    boxAux = QtWidgets.QVBoxLayout(self.lvrv_frame)
                    self.lvrv = FigureLVRV(df, self.lvrv_frame, lvrvType, None, width=6, height=5, dpi=100)
                    boxAux.addWidget(self.lvrv)
                else:
                    self.lvrv.updatelvrv(df)
        except Exception as e:
            print(e)
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: Please import a valid LV-RV experiment CSV')
            error_dialog.exec()

    def exportPCA(self):
        print("hacer")
    def importPCA(self):
        print("hacer")

    # data base functions
    def addEntrydb(self, strain, repetition, fvp, lvp, expName, params, result):
        fvp = str(fvp)
        lvp = str(lvp)
        vevExp = {"_id": {"user": self.namePC, "strain": strain, "repetition": repetition,"fvp":fvp,
                          "lvp":lvp,"expName":expName},"params":params,"result":result}
        vevExperiments = self.db['ResultsExperiments']


        entryD = vevExperiments.find_one({"_id": vevExp["_id"]})
        if entryD == None: #insert
            vevExperiments.insert_one(vevExp)
        else: #update
            vevExperiments.update_one({'_id': vevExp["_id"]}, {"$set": vevExp}, upsert=False)

    def addUser(self):
        if not self.db.users.find({'_id': {"$in": [self.namePC]}}).count() > 0:
            userCollection = self.db['users']
            user = {"_id":self.namePC}
            userCollection.insert_one(user)

def main():
    app = QtWidgets.QApplication([])
    win = Interfaz()
    win.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()

